use std::path::PathBuf;

use clap::{
    builder::{
        styling::{AnsiColor, Color, Style},
        Styles,
    },
    Parser,
};

#[derive(Debug, Parser)]
#[command(styles=get_styles())]
pub struct Cli {
    /// The serial port to use
    #[clap(short, long, value_name = "PORT")]
    pub port: Option<String>,

    /// baud rate
    #[clap(short, long, default_value = "115200")]
    pub baud_rate: u32,

    /// disable SYNC MODE (wait for a sync signal from the device before proceding)
    #[clap(short, long)]
    pub no_sync: bool,

    /// The low rate telemetry CSV file to read from
    #[clap(value_name = "LOW_RATE_FILE")]
    pub low_rate_file: Option<PathBuf>,

    /// The high rate telemetry CSV file to read from
    #[clap(value_name = "HIGH_RATE_FILE")]
    pub high_rate_file: Option<PathBuf>,

    /// The log level
    #[clap(short, long)]
    pub verbose: bool,

    /// The flight radar API usage
    #[clap(short, long)]
    pub fr_api: Option<String>,
}

pub fn get_styles() -> Styles {
    Styles::styled()
        .usage(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .header(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .literal(Style::new().fg_color(Some(Color::Ansi(AnsiColor::Green))))
        .invalid(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .error(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .valid(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Green))),
        )
        .placeholder(Style::new().fg_color(Some(Color::Ansi(AnsiColor::White))))
}
