use std::io::{Read, Result};

use serde::Deserialize;
use skyward_mavlink::lyra::{MavMessage, ROCKET_FLIGHT_TM_DATA, ROCKET_STATS_TM_DATA};

use crate::packet::TimedMessage;

#[derive(Debug, Deserialize)]
struct Lrtm {
    timestamp: u64,
    latitude: f64,
    longitude: f64,
    altitude: f64,
}

#[derive(Debug, Deserialize)]
struct Hrtm {
    timestamp: u64,
    n: f64,
    e: f64,
    d: f64,
    vn: f64,
    ve: f64,
    vd: f64,
}

/// Read low rate telemetry data from a CSV file
pub fn read_low_rate_telemetry(reader: impl Read) -> Result<Vec<TimedMessage>> {
    let mut packets = Vec::new();
    let mut rdr = csv::Reader::from_reader(reader);
    for result in rdr.deserialize() {
        let r: Lrtm = result?;
        let msg_data = ROCKET_STATS_TM_DATA {
            ref_lat: r.latitude as f32,
            ref_lon: r.longitude as f32,
            ref_alt: r.altitude as f32,
            ..Default::default()
        };
        let mav_msg = MavMessage::ROCKET_STATS_TM(msg_data);
        packets.push(TimedMessage::new(r.timestamp, mav_msg));
    }
    Ok(packets)
}

/// Read high rate telemtry data from a CSV file
pub fn read_high_rate_telemetry(reader: impl Read) -> Result<Vec<TimedMessage>> {
    let mut packets = Vec::new();
    let mut rdr = csv::Reader::from_reader(reader);
    for result in rdr.deserialize() {
        let r: Hrtm = result?;
        let msg_data = ROCKET_FLIGHT_TM_DATA {
            timestamp: r.timestamp,
            nas_n: r.n as f32,
            nas_e: r.e as f32,
            nas_d: r.d as f32,
            nas_vn: r.vn as f32,
            nas_ve: r.ve as f32,
            nas_vd: r.vd as f32,
            ..Default::default()
        };
        let mav_msg = MavMessage::ROCKET_FLIGHT_TM(msg_data);
        packets.push(TimedMessage::new(r.timestamp, mav_msg));
    }
    Ok(packets)
}
